@@ -3,6 +3,7 @@ from uuid import uuid4
 
 from django.conf import settings
 from django.utils.translation import gettext_lazy as _
+from django.utils.module_loading import import_string
 
 from .exceptions import TokenBackendError, TokenError
 from .settings import api_settings
@@ -35,7 +36,7 @@ class Token:
         # Set up token
         if token is not None:
             # An encoded token was provided
-            from .state import token_backend
+            token_backend = self.get_token_backend()
 
             # Decode token
             try:
@@ -77,9 +78,7 @@ class Token:
         """
         Signs and returns a token as a base64 encoded string.
         """
-        from .state import token_backend
-
-        return token_backend.encode(self.payload)
+        return self.get_token_backend().encode(self.payload)
 
     def verify(self):
         """
@@ -166,6 +165,15 @@ class Token:
         token[api_settings.USER_ID_CLAIM] = user_id
 
         return token
+    
+    _token_backend = None
+    
+    def get_token_backend(self):
+        if self._token_backend is None:
+            self._token_backend = import_string(
+                "rest_framework_simplejwt.state.token_backend"
+            )
+        return self._token_backend
 
 
 class BlacklistMixin:
